/*
 * Heap.java:  Nov 3, 2006
 * Changxi Zheng (cxzheng@cs.cornell.edu)
 */
package cornell.cs211;
 
import java.util.Collection;

/**
 * This interface for <tt>Heap</tt>, integrated into the java Collection
 * framwork. Besides basic {@link java.util.Collection Collection} operations, 
 * <tt>Heap</tt> provide additional {@link #pop}, {@link #push}, and {@link #top}
 * operations.
 * 
 * @see AbstractHeap
 * @since 1.5
 * @author Changxi Zheng 
 * @param <E> The type of the elements held in this heap.
 */
public interface Heap<E> extends Collection<E> {
    /**
     * Retrieves and removes the head of this heap if exsits. Otherwise,
     * do nothing and return <tt>null</tt>.
     * 
     * @return the head of the heap, or <tt>null</tt> if this heap is empty.
     */
    E pop();

    /**
     * Retrieves but doesn't remove the head of this heap, or <tt>null</tt>
     * if this heap is empty.
     * 
     * @return the head of the heap, or <tt>null</tt> if this heap is empty.
     */
    E top();
    
    /**
     * Insert the specified element into the heap. This method is generally 
     * preferable to method {@link Collection#add}.
     * 
     * @param o the element to insert
     * @return <tt>true</tt> if it adds the element in the heap successfully,
     *         <tt>false</tt> otherwise.
     */
    boolean push(E o);
}
